import json
import logging
from argparse import ArgumentParser
from pathlib import Path
from pprint import pprint

from mediacatch.commands import BaseCLICommand
from mediacatch.speech import upload, wait_for_result

logger = logging.getLogger('mediacatch.cli.speech')


def speech_cli_factory(args):
    return SpeechCLI(
        file_path=args.file_path,
        save_result=args.save_result,
        qouta=args.quota,
        fallback_language=args.fallback_language,
        only_upload=args.only_upload,
        url=args.url,
        max_threads=args.max_threads,
        max_request_retries=args.max_request_retries,
        request_delay=args.request_delay,
        chunk_size=args.chunk_size,
        compress_input=args.compress_input,
        sample_rate=args.sample_rate,
        get_result_timeout=args.get_result_timeout,
        get_result_delay=args.get_result_delay,
    )


class SpeechCLI(BaseCLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        speech_parser: ArgumentParser = parser.add_parser(
            'speech', help='CLI tool to run inference with MediaCatch Speech API'
        )

        speech_parser.add_argument(
            'file_path', type=str, help='Path to the file to run inference on'
        )
        speech_parser.add_argument(
            '--save-result',
            type=str,
            default=None,
            help='Save result to a file',
        )
        speech_parser.add_argument(
            '--quota',
            type=str,
            default=None,
            help='Quota for the speech recognition',
        )
        speech_parser.add_argument(
            '--fallback-language',
            type=str,
            default=None,
            help='Fallback language for the speech recognition',
        )
        speech_parser.add_argument(
            '--only-upload',
            action='store_true',
            help='Only upload the file to MediaCatch Speech API',
        )
        speech_parser.add_argument(
            '--url',
            type=str,
            default='https://s2t.mediacatch.io/api/v2',
            help='URL to the MediaCatch Speech API',
        )
        speech_parser.add_argument(
            '--max-threads',
            type=int,
            default=5,
            help='Maximum number of threads to use for uploading and waiting for result',
        )
        speech_parser.add_argument(
            '--max-request-retries',
            type=int,
            default=3,
            help='Maximum number of retries for requests to the API',
        )
        speech_parser.add_argument(
            '--request-delay',
            type=float,
            default=1.0,
            help='Delay between requests to the API',
        )
        speech_parser.add_argument(
            '--chunk-size',
            type=int,
            default=100 * 1024 * 1024,
            help='Chunk size for uploading the file to the API',
        )
        speech_parser.add_argument(
            '--compress-input',
            action='store_true',
            help='Compress the input file to OGG format (requires FFMPEG >= 6.1)',
        )
        speech_parser.add_argument(
            '--sample-rate',
            type=int,
            default=16000,
            help='Sample rate of the audio file (only used if compress-input is set)',
        )
        speech_parser.add_argument(
            '--get-result-timeout',
            type=int,
            default=3600,
            help='Timeout for waiting for the result in seconds',
        )
        speech_parser.add_argument(
            '--get-result-delay',
            type=float,
            default=10.0,
            help='Delay between requests to get the result',
        )

        speech_parser.set_defaults(func=speech_cli_factory)

    def __init__(
        self,
        file_path: str,
        save_result: str | None = None,
        qouta: str | None = None,
        fallback_language: str | None = None,
        only_upload: bool = False,
        url: str = 'https://s2t.mediacatch.io/api/v2',
        max_threads: int = 5,
        max_request_retries: int = 3,
        request_delay: float = 1.0,
        chunk_size: int = 100 * 1024 * 1024,
        compress_input: bool = False,
        sample_rate: int = 16000,
        get_result_timeout: int = 3600,
        get_result_delay: float = 10.0,
    ) -> None:
        self.file_path = file_path
        self.save_result = save_result
        self.qouta = qouta
        self.fallback_language = fallback_language
        self.only_upload = only_upload
        self.url = url
        self.max_threads = max_threads
        self.max_request_retries = max_request_retries
        self.request_delay = request_delay
        self.chunk_size = chunk_size
        self.compress_input = compress_input
        self.sample_rate = sample_rate
        self.get_result_timeout = get_result_timeout
        self.get_result_delay = get_result_delay

    def run(self) -> None:
        # Upload file to MediaCatch Speech API
        file_id = upload(
            self.file_path,
            quota=self.qouta,
            fallback_language=self.fallback_language,
            max_threads=self.max_threads,
            max_request_retries=self.max_request_retries,
            request_delay=self.request_delay,
            chunk_size=self.chunk_size,
            url=self.url,
            compress_input=self.compress_input,
            sample_rate=self.sample_rate,
        )

        if self.only_upload:
            result_url = f'{self.url}/result/{file_id}'
            logger.info(f'File uploaded with ID {file_id}, get result at {result_url}')
            return

        # Wait for result
        result = wait_for_result(
            file_id, url=self.url, timeout=self.get_result_timeout, delay=self.get_result_delay
        )
        if not result:
            logger.error(f'Failed to get result for file {self.file_path}')
            return

        logger.info('Result:')
        pprint(result)

        # Save result to file
        if self.save_result:
            Path(self.save_result).write_text(json.dumps(result, indent=4))
            logger.info(f'Result saved to {self.save_result}')



import json
import logging
from argparse import ArgumentParser
from pathlib import Path
from pprint import pprint

from mediacatch.commands import BaseCLICommand
from mediacatch.vision import upload, wait_for_result

logger = logging.getLogger('mediacatch.cli.vision')


def vision_cli_factory(args):
    return VisionCLI(
        file_path=args.file_path,
        type=args.type,
        save_result=args.save_result,
        fps=args.fps,
        tolerance=args.tolerance,
        min_levensthein_ratio=args.min_levensthein_ratio,
        min_bbox_iou=args.min_bbox_iou,
        min_text_confidence=args.min_text_confidence,
        max_text_confidence=args.max_text_confidence,
        max_text_length=args.max_text_length,
        moving_text_threshold=args.moving_text_threshold,
        max_height_width_ratio=args.max_height_width_ratio,
        get_detection_histogram=args.get_detection_histogram,
        detection_histogram_bins=args.detection_histogram_bins,
        max_height_difference_ratio=args.max_height_difference_ratio,
        max_horizontal_distance_ratio=args.max_horizontal_distance_ratio,
        only_upload=args.only_upload,
    )


class VisionCLI(BaseCLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        vision_parser: ArgumentParser = parser.add_parser(
            'vision', help='CLI tool to run inference with MediaCatch Vision API'
        )
        vision_parser.add_argument(
            'type',
            type=str,
            choices=['ocr', 'face'],
            help='Type of inference to run on the file',
        )
        vision_parser.add_argument(
            'file_path', type=str, help='Path to the file to run inference on'
        )
        vision_parser.add_argument(
            '--save-result',
            type=str,
            default=None,
            help='Save result to a file',
        )
        vision_parser.add_argument(
            '--fps',
            type=int,
            default=None,
            help='FPS for the OCR results',
        )
        vision_parser.add_argument(
            '--tolerance',
            type=int,
            default=None,
            help='Tolerance in seconds for merging text detection for OCR',
        )
        vision_parser.add_argument(
            '--min-levensthein-ratio',
            type=float,
            default=None,
            help='Minimum Levenshtein ratio for merging text detection for OCR (more info here: https://rapidfuzz.github.io/Levenshtein/levenshtein.html#ratio)',
        )
        vision_parser.add_argument(
            '--min-bbox-iou',
            type=float,
            default=None,
            help='Minimum bounding box intersection over union for merging text detection for OCR',
        )
        vision_parser.add_argument(
            '--max-text-length',
            type=int,
            default=None,
            help='If text length is less than this value, use max_text_confidence as confidence threshold for OCR',
        )
        vision_parser.add_argument(
            '--min-text-confidence',
            type=float,
            default=None,
            help='Confidence threshold for text detection for OCR (if text length is greater than max_text_length)',
        )
        vision_parser.add_argument(
            '--max-text-confidence',
            type=float,
            default=None,
            help='Confidence threshold for text detection for OCR (if text length is less than max_text_length)',
        )
        vision_parser.add_argument(
            '--moving-text-threshold',
            type=int,
            default=None,
            help='If merged text detections center moves more pixels than this threshold, it will be considered moving text for OCR',
        )
        vision_parser.add_argument(
            '--max-height-width-ratio',
            type=float,
            default=None,
            help='Discard detection if height/width ratio is greater than this value',
        )
        vision_parser.add_argument(
            '--get-detection-histogram',
            action='store_true',
        )
        vision_parser.add_argument(
            '--detection-histogram-bins',
            type=int,
            default=None,
            help='Number of bins for histogram calculation',
        )
        vision_parser.add_argument(
            '--max-height-difference-ratio',
            type=float,
            default=None,
            help='Determine the maximum allowed difference in height between two text boxes for them to be merged',
        )
        vision_parser.add_argument(
            '--max-horizontal-distance-ratio',
            type=float,
            default=None,
            help='Determine if two boxes are close enough horizontally to be considered part of the same text line',
        )
        vision_parser.add_argument(
            '--only-upload',
            action='store_true',
            help='Only upload the file to MediaCatch Vision API',
        )
        vision_parser.set_defaults(func=vision_cli_factory)

    def __init__(
        self,
        file_path: str,
        type: str,
        save_result: str | None = None,
        fps: int | None = None,
        tolerance: int | None = None,
        min_levensthein_ratio: float | None = None,
        min_bbox_iou: float | None = None,
        min_text_confidence: float | None = None,
        max_text_confidence: float | None = None,
        max_text_length: int | None = None,
        moving_text_threshold: int | None = None,
        max_height_width_ratio: float | None = None,
        get_detection_histogram: bool | None = None,
        detection_histogram_bins: int | None = None,
        max_height_difference_ratio: float | None = None,
        max_horizontal_distance_ratio: float | None = None,
        only_upload: bool = False,
    ) -> None:
        self.file_path = file_path
        self.type = type
        self.save_result = save_result
        self.fps = fps
        self.tolerance = tolerance
        self.min_levensthein_ratio = min_levensthein_ratio
        self.min_bbox_iou = min_bbox_iou
        self.min_text_confidence = min_text_confidence
        self.max_text_confidence = max_text_confidence
        self.max_text_length = max_text_length
        self.moving_text_threshold = moving_text_threshold
        self.max_height_width_ratio = max_height_width_ratio
        self.get_detection_histogram = get_detection_histogram
        self.detection_histogram_bins = detection_histogram_bins
        self.max_height_difference_ratio = max_height_difference_ratio
        self.max_horizontal_distance_ratio = max_horizontal_distance_ratio
        self.only_upload = only_upload

    def run(self) -> None:
        # Upload file to MediaCatch Vision API
        file_id = upload(
            self.file_path,
            self.type,
            fps=self.fps,
            tolerance=self.tolerance,
            min_bbox_iou=self.min_bbox_iou,
            min_levenshtein_ratio=self.min_levensthein_ratio,
            moving_threshold=self.moving_text_threshold,
            max_text_length=self.max_text_length,
            min_text_confidence=self.min_text_confidence,
            max_text_confidence=self.max_text_confidence,
            max_height_width_ratio=self.max_height_width_ratio,
            get_detection_histogram=self.get_detection_histogram,
            detection_histogram_bins=self.detection_histogram_bins,
            max_height_difference_ratio=self.max_height_difference_ratio,
            max_horizontal_distance_ratio=self.max_horizontal_distance_ratio,
        )
        if self.only_upload:
            logger.info(f'Find result at https://api.mediacatch.io/vision/result/{file_id}')
            return

        # Wait for result
        result = wait_for_result(file_id)
        if not result:
            logger.error('Failed to get result from MediaCatch Vision API')
            return

        logger.info('Results:')
        pprint(result)

        # Save result to a file
        if self.save_result:
            Path(self.save_result).write_text(json.dumps(result, indent=4))
            logger.info(f'Result saved to {self.save_result}')



import logging
import sys
import os

from dotenv import load_dotenv

logging.basicConfig(
    format='[%(asctime)s][%(levelname)s][%(name)s] %(message)s',
    datefmt='%d/%m/%Y-%H:%M:%S',
    handlers=[logging.StreamHandler(sys.stdout)],
    level=logging.INFO,
)

load_dotenv()
mediacatch_api_key = os.environ['MEDIACATCH_API_KEY']

__version__ = '0.3.1'



import logging
import tempfile
from concurrent.futures import ThreadPoolExecutor, as_completed
from pathlib import Path

import requests

from mediacatch import mediacatch_api_key
from mediacatch.utils import (
    MediacatchUploadError,
    check_ffmpeg,
    compress_to_ogg,
    make_request,
    read_file_in_chunks,
)

logger = logging.getLogger('mediacatch.speech.upload')


def upload(
    fpath: str | Path,
    quota: str | None = None,
    fallback_language: str | None = None,
    max_threads: int = 5,
    max_request_retries: int = 3,
    request_delay: float = 0.5,
    chunk_size=100 * 1024 * 1024,  # 100 MB
    url: str = 'https://s2t.mediacatch.io/api/v2',
    compress_input: bool = False,
    sample_rate: int = 16000,
    verbose: bool = True,
) -> str:
    """Uploads a file to MediaCatch Speech API.

    Args:
        fpath (str | Path): Path to the file to upload.
        quota (str | None, optional): The quota to bill transcription hours from. Can be None if the user only has one quota. Defaults to None.
        fallback_language (str | None, optional): Overrides the language to transcribe in if language identification fails. If None, uses the default language of the quota. Defaults to None.
        max_threads (int, optional): Number of maximum threads. Defaults to 5.
        max_request_retries (int, optional): Number of maximum retries for request. Defaults to 3.
        request_delay (float, optional): Delay between request retries. Defaults to 0.5.
        chunk_size (_type_, optional): Size of each chunk to upload. Defaults to 100*1024*1024.
        url (str, optional): URL of the MediaCatch Speech API. Defaults to 'https://s2t.mediacatch.io/api/v2'.
        compress_input (bool, optional): Compress the input file to OGG format (Requires FFMPEG >= 6.1). Defaults to False.
        sample_rate (int, optional): Sample rate of the audio file. Defaults to 16000.
        verbose (bool, optional): Show verbose output. Defaults to True.

    Returns:
        str: File ID of the uploaded file.
    """
    if not isinstance(fpath, Path):
        fpath = Path(fpath)

    if not fpath.is_file():
        raise MediacatchUploadError(f'File {fpath} does not exist')

    if compress_input:
        check_ffmpeg()

        if verbose:
            logger.info(f'Compressing file {fpath} to OGG format')

        with tempfile.NamedTemporaryFile(suffix='.ogg', delete=False) as temp_file:
            temp_path = Path(temp_file.name)

        compress_to_ogg(str(fpath), str(temp_path), sample_rate=sample_rate)
    else:
        temp_path = None

    if verbose:
        logger.info(f'Uploading file {fpath} to MediaCatch Speech API')

    headers = {
        'Content-type': 'application/json',
        'X-API-KEY': mediacatch_api_key,
        'X-Quota': str(quota),
    }

    # Initiate file upload
    start_upload_url = f'{url}/upload/'
    data = {
        'file_name': fpath.with_suffix('.ogg').name if compress_input else fpath.name,
        'file_extension': '.ogg' if compress_input else fpath.suffix,
        'quota': quota,
        'fallback_language': fallback_language,
    }
    response = make_request(
        'post',
        start_upload_url,
        headers=headers,
        max_retries=max_request_retries,
        delay=request_delay,
        json=data,
    )
    file_id = response.json()['file_id']

    # Upload file chunks
    upload_file_url = f'{url}/upload/{{file_id}}/{{part_number}}'
    etags = []

    def upload_chunk(part_number: int, chunk: bytes) -> None:
        # Get signed URL to upload chunk
        signed_url_response = make_request(
            'get',
            upload_file_url.format(file_id=file_id, part_number=part_number),
            headers=headers,
        )
        signed_url = signed_url_response.json()['url']

        # Upload chunk to storage
        reponse = requests.put(signed_url, data=chunk)
        etag = reponse.headers['ETag']
        etags.append({'e_tag': etag, 'part_number': part_number})

    with (
        ThreadPoolExecutor(max_workers=max_threads) as executor,
        temp_path.open('rb') if compress_input else fpath.open('rb') as f,
    ):
        futures = {
            executor.submit(upload_chunk, part_number, chunk): part_number
            for part_number, chunk in enumerate(
                read_file_in_chunks(file_=f, chunk_size=chunk_size), start=1
            )
        }

        for future in as_completed(futures):
            part_number = futures[future]
            try:
                future.result()
            except Exception as e:
                if verbose:
                    logger.error(f'Chunk {part_number} failed to upload due to: {e}')

    # Complete file upload
    complete_upload_url = f'{url}/upload/{file_id}/complete'
    response = make_request('post', complete_upload_url, json={'parts': etags}, headers=headers)
    estimated_processing_time = response.json()['estimated_processing_time']

    if verbose:
        logger.info(
            f'Compressed file {fpath} uploaded successfully. Estimated processing time: {estimated_processing_time}'
        )

    if temp_path and isinstance(temp_path, Path):
        temp_path.unlink()

    return file_id



import json
import logging
import time
from typing import Any

import requests

from mediacatch.utils import MediacatchAPIError, MediacatchError, MediacatchTimeoutError

logger = logging.getLogger('mediacatch.speech.result')


def wait_for_result(
    file_id: str,
    url: str = 'https://s2t.mediacatch.io/api/v2',
    timeout: int = 3600,
    delay: int = 10,
    verbose: bool = True,
) -> dict[str, Any] | None:
    """Wait for result from a URL.

    Args:
        file_id (str): The file ID to get the result from.
        url (str): The URL to get the result from.
        timeout (int, optional): Timeout for waiting in seconds. Defaults to 3600.
        delay (int, optional): Delay between each request. Defaults to 10.
        verbose (bool, optional): Show verbose output. Defaults to True.

    Returns:
        dict[str, Any] | None: Dictionary with the result from the URL or None if failed.
    """
    result_url = f'{url}/result/{file_id}'
    if verbose:
        logger.info(f'Waiting for result from {result_url}')
    start_time = time.time()
    end_time = start_time + timeout
    while time.time() < end_time:
        try:
            response = requests.get(result_url)
            if response.status_code in [404, 500]:
                reason = (
                    'File not found' if response.status_code == 404 else 'Internal server error'
                )
                raise MediacatchAPIError(response.status_code, reason)
            elif response.status_code in [425]:
                time.sleep(delay)
                continue

            response.raise_for_status()
            result = response.json()
            elapsed_time = time.time() - start_time
            if verbose:
                logger.info(f'Got result from {result_url} in {elapsed_time:.2f} seconds')

            return result

        except (requests.RequestException, json.JSONDecodeError):
            if verbose:
                logger.error('Error occurred while waiting for JSON response')

        except Exception as e:
            if verbose:
                logger.error(f'Failed to get result from {result_url}: {e}')

            raise MediacatchError(f'Failed to get result from {result_url}: {e}')

        time.sleep(delay)

    if verbose:
        logger.error(f'Timeout waiting for result from {result_url}, give up')

    raise MediacatchTimeoutError(f'Timeout waiting for result from {result_url}')


if __name__ == '__main__':
    from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser
    from pprint import pprint

    parser = ArgumentParser(
        description='Wait for result from a URL',
        formatter_class=ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        'file_id',
        type=str,
        help='The file ID to get the result from',
    )
    parser.add_argument(
        '--url',
        type=str,
        default='https://s2t.mediacatch.io/api/v2',
        help='The URL to get the result from',
    )
    parser.add_argument(
        '--timeout',
        type=int,
        default=3600,
        help='Timeout for waiting in seconds',
    )
    parser.add_argument(
        '--delay',
        type=int,
        default=10,
        help='Delay between each request',
    )
    parser.add_argument(
        '--verbose',
        action='store_true',
        help='Show verbose output',
    )
    args = parser.parse_args()

    result = wait_for_result(
        args.file_id, url=args.url, timeout=args.timeout, delay=args.delay, verbose=args.verbose
    )
    pprint(result)



import logging
import subprocess
from datetime import timedelta
from pathlib import Path
from typing import Any

from srt import Subtitle, compose
from tqdm.auto import tqdm

logger = logging.getLogger('mediacatch.viz.speech')


class SpeechViz:
    """Class to visualize speech results."""

    def __init__(
        self,
        file_path: str | Path,
        results: dict[str, Any],
        output_path: str | Path,
        subtitles: bool = True,
        meta: bool = True,
        max_subtitle_length: int = 10,
        max_chars_in_subtitle: int = 100,
    ) -> None:
        """Visualize speech results.

        Args:
            file_path (str | Path): Path to file.
            results (dict[str, Any]): Results from speech API.
            output_path (str | Path): Output path.
            subtitles (bool, optional): If true, add subtitles. Defaults to True.
            meta (bool, optional): If true, add meta data. Defaults to True.
            max_subtitle_length (int, optional): Maximum length of subtitle. Defaults to 10.
            max_chars_in_subtitle (int, optional): Maximum characters in subtitle. Defaults to 100.
        """
        if not isinstance(file_path, Path):
            file_path = Path(file_path)

        if not isinstance(output_path, Path):
            output_path = Path(output_path)

        assert file_path.is_file(), f'File not found: {file_path}'
        assert results

        self.file_path = file_path
        self.results = results
        self.output_path = output_path
        self.subtitles = subtitles
        self.meta = meta
        self.max_subtitle_length = max_subtitle_length
        self.max_chars_in_subtitle = max_chars_in_subtitle

    def create_viz(self) -> None:
        """Create visualization."""
        utterances = self.results['result']['utterances']

        text_srt_file = None
        if self.subtitles:
            text_subtitles = self.create_text_subtitle(utterances)
            text_srt_file = self.write_srt_file(self.output_path, '.text.srt', text_subtitles)

        meta_srt_file = None
        if self.meta:
            meta_subtitles = self.create_meta_subtitle(utterances)
            meta_srt_file = self.write_srt_file(self.output_path, '.meta.srt', meta_subtitles)

        self.burn_subtitles(text_srt_file, meta_srt_file)

    def create_text_subtitle(self, utterances: list[dict[str, Any]]) -> str:
        """Create text subtitles.

        Args:
            utterances (list[dict[str, Any]]): Utterances for speech results.

        Returns:
            str: Subtitles in SRT format.
        """
        subtitles = []
        current_subtitle = []
        sub_start = 0
        sub_end = 0
        for utterance in tqdm(utterances, desc='Creating text subtitles'):
            for word_data in utterance.get('words', []):
                word = word_data.get('word', '')
                i_start = word_data.get('start', 0)
                i_end = word_data.get('end', 0)

                if not current_subtitle:
                    sub_start = i_start

                current_subtitle.append(word)
                sub_end = i_end

                if self._should_end_subtitle(current_subtitle, sub_start, sub_end, word):
                    subtitles.append((' '.join(current_subtitle), sub_start, sub_end))
                    current_subtitle = []

        if current_subtitle:
            subtitles.append((' '.join(current_subtitle), sub_start, sub_end))

        return compose(
            [
                Subtitle(
                    index=i,
                    start=timedelta(seconds=start),
                    end=timedelta(seconds=end),
                    content=text,
                )
                for i, (text, start, end) in enumerate(subtitles)
            ]
        )

    def _should_end_subtitle(
        self, words: list[str], start: float, end: float, last_word: str
    ) -> bool:
        """Check if subtitle should end.

        Args:
            words (list[str]): List of words in subtitle.
            start (float): Start time of subtitle.
            end (float): End time of subtitle.
            last_word (str): Last word in subtitle.

        Returns:
            bool: True if subtitle should end else False.
        """
        return (
            end - start >= self.max_subtitle_length
            or len(' '.join(words)) >= self.max_chars_in_subtitle
            or last_word[-1] in ['.', '!', '?']
        )

    def create_meta_subtitle(self, utterances: list[dict[str, Any]]) -> str:
        """Create meta subtitles.

        Args:
            utterances (list[dict[str, Any]]): Utterances for speech results.

        Returns:
            str: Subtitles in SRT format.
        """
        subs = []
        for i, utterance in tqdm(enumerate(utterances), desc='Creating meta subtitles'):
            meta = utterance.get('meta', {})
            if not meta:
                continue

            sub_parts = [meta.get('speaker', 'Unknown').capitalize()]

            for attribute in ['language', 'gender']:
                attr_data = meta.get(attribute, {})
                if attr_data:
                    label = attr_data.get('label')
                    confidence = attr_data.get('confidence')
                    if label:
                        sub_parts.append(f'{label.capitalize()}')
                    if confidence:
                        sub_parts.append(f'{confidence * 100:.02f}%')

            sub = '\t\t'.join(sub_parts)

            start_second = utterance.get('start')
            end_second = utterance.get('end')
            if start_second is not None and end_second is not None:
                subs.append(
                    Subtitle(
                        index=i,
                        start=timedelta(seconds=start_second),
                        end=timedelta(seconds=end_second),
                        content=sub,
                    )
                )

        return compose(subs)

    @staticmethod
    def write_srt_file(input_path: str, output_suffix: str, subtitles: str) -> Path:
        """Write subtitles to file as SRT.

        Args:
            input_path (str): Input path.
            output_suffix (str): Output suffix.
            subtitles (str): Subtitles in SRT format.

        Returns:
            Path: Output path.
        """
        output_path = Path(input_path).with_suffix(output_suffix)
        output_path.write_text(subtitles)
        return output_path

    def burn_subtitles(
        self, text_srt_file: Path | None, meta_srt_file: Path | None, stream: bool = False
    ) -> None:
        """Burn subtitles to video.

        Args:
            text_srt_file (Path | None): Subtitles in SRT format.
            meta_srt_file (Path | None): Meta data in SRT format.
            stream (bool, optional): Set text style to stream. Defaults to False.
        """
        logger.info(f'Burning subtitles to {self.output_path}')

        text_style = "'BackColour=&H40000000,BorderStyle=4'" + (',Alignment=1' if stream else '')
        meta_style = "'Alignment=6,BackColour=&H40000000,BorderStyle=4'"

        ffmpeg_commands = []

        if text_srt_file and meta_srt_file:
            tmp_file = self.output_path.with_name('tmp.mkv')
            ffmpeg_commands = [
                self._create_ffmpeg_command(self.file_path, tmp_file, text_srt_file, text_style),
                self._create_ffmpeg_command(tmp_file, self.output_path, meta_srt_file, meta_style),
            ]
        elif text_srt_file:
            ffmpeg_commands = [
                self._create_ffmpeg_command(
                    self.file_path, self.output_path, text_srt_file, text_style
                )
            ]
        elif meta_srt_file:
            ffmpeg_commands = [
                self._create_ffmpeg_command(
                    self.file_path, self.output_path, meta_srt_file, text_style
                )
            ]

        for command in ffmpeg_commands:
            subprocess.run(command, shell=True, check=True)

        if len(ffmpeg_commands) > 1:
            tmp_file.unlink()

    def _create_ffmpeg_command(
        self, input_file: Path, output_file: Path, srt_file: Path, style: str
    ) -> str:
        """Create ffmpeg command.

        Args:
            input_file (Path): Input file.
            output_file (Path): Output file.
            srt_file (Path): SRT file.
            style (str): Style for subtitles.

        Returns:
            str: ffmpeg command.
        """
        return f'ffmpeg -hide_banner -loglevel error -stats -i {input_file} -vf "subtitles={srt_file}:force_style={style}" {output_file}'



import logging
import os
from typing import Any

from mediacatch.viz.annotator import Annotator
from mediacatch.viz.base_viz import BaseViz

logger = logging.getLogger('mediacatch.viz.ocr')


class OCRViz(BaseViz):
    """Class to visualize OCR results."""

    def __init__(
        self,
        file_path: str,
        results: list[dict[str, Any]],
        output_path: str,
        max_text_length: int = 30,
        avg_char_width: int = 10,
        min_chars: int = 3,
    ) -> None:
        """Initialize OCRViz.

        Args:
            file_path (str): File path.
            results (list[dict[str, Any]]): Results from OCR API.
            output_path (str): Output path.
            max_text_length (int, optional): Max text length to show (will add ... at the end). Defaults to 30.
            avg_char_width (int, optional): Average character. Defaults to 10.
            min_chars (int, optional): Minimum characters to alway show. Defaults to 3.
        """
        assert os.path.isfile(file_path), f'File not found: {file_path}'
        assert results, 'No OCR results found'
        assert max_text_length > 0, 'max_text_length must be greater than 0'

        self.file_path = file_path
        self.results = results
        self.results = sorted(self.results, key=lambda x: x['start_frame_idx'])
        self.output_path = output_path
        self.max_text_length = max_text_length
        self.avg_char_width = avg_char_width
        self.min_chars = min_chars

    def draw_results(self, anno: Annotator, frame_idx: int, width: int, height: int) -> None:
        for r in self.results:
            start_frame = r['start_frame_idx']
            end_frame = r['end_frame_idx']

            if start_frame <= frame_idx <= end_frame:
                x, y, w, h = r['bbox']
                bbox = [x, y, x + w, y + h]
                text = r['text']
                max_chars = max(self.min_chars, w // self.avg_char_width)
                text = text if len(text) <= max_chars else text[:max_chars] + '...'
                anno.bbox_label(
                    bbox,
                    label=text,
                )



import logging
import os
from typing import Any

import numpy as np

from mediacatch.viz.annotator import Annotator
from mediacatch.viz.base_viz import BaseViz

logger = logging.getLogger('mediacatch.viz.detect')


def color_gradient(
    start_color: tuple[int, int, int], end_color: tuple[int, int, int], steps: int
) -> list:
    """Create a color gradient.

    Args:
        start_color (tuple[int, int, int]): Start color (r, g, b).
        end_color (tuple[int, int, int]): End color (r, g, b).
        steps (int): Number of steps.

    Returns:
        list: List of colors (r, g, b).
    """
    start = np.array(start_color)
    end = np.array(end_color)
    colors = [start + (end - start) * i / (steps - 1) for i in range(steps)]
    return [(int(r), int(g), int(b)) for r, g, b in colors]


class DetectViz(BaseViz):
    """Class to visualize detection results."""

    _color1 = (101, 16, 50)
    _color2 = (250, 139, 172)

    def __init__(
        self,
        file_path: str,
        results: list[dict[str, Any]],
        output_path: str,
        color: tuple[int, int, int] | None = None,
        min_conf: float = 0.5,
        classes: list[str] = None,
    ) -> None:
        """Initialize DetectViz.

        Args:
            file_path (str): Path to file.
            results (list[dict[str, Any]]): Results from Medicatch Vision API.
            output_path (str): Output path.
            color (tuple[int, int, int] | None, optional): Set a fixed color for all bounding boxes (r, g, b). Defaults to None.
            min_conf (float, optional): Minium confidence threshold. Defaults to 0.5.
            classes (list[str], optional): List of classes to overwrite model classes labels. Defaults to None.
        """
        assert os.path.isfile(file_path), f'File not found: {file_path}'
        assert results, 'No detections results found'

        self.file_path = file_path
        self.results = results
        self.results = sorted(self.results, key=lambda x: x['frame'])
        self.output_path = output_path
        self.color = color
        self.min_conf = min_conf
        self.classes = classes

        if not self.classes:
            self.classes = list(set(r['label'] for r in self.results))

        self.colors = (
            [(236, 56, 131)]
            if len(self.classes) == 1
            else color_gradient(self._color1, self._color2, len(self.classes))
        )

    def draw_results(self, anno: Annotator, frame_idx: int, width: int, height: int) -> None:
        for r in self.results:
            frame = r['frame']
            conf = r['conf']
            label: str = r['label']
            if frame == frame_idx and conf > self.min_conf and label in self.classes:
                x1, y1, x2, y2 = r['bbox']
                x1 = int(x1 * width)
                y1 = int(y1 * height)
                x2 = int(x2 * width)
                y2 = int(y2 * height)
                bbox = [x1, y1, x2, y2]
                color = self.colors[self.classes.index(label)]
                # Convert redbull labels and colors
                if label == 'rb_at':
                    label = 'AlphaTauri'
                    color = (236, 56, 131)
                elif label == 'rb_wfl':
                    label = 'Wings for Life'
                    color = (236, 56, 131)
                elif label.startswith('rb_'):
                    label = 'Red Bull'
                    color = (236, 56, 131)
                elif label == 'danske_spil_oddset':
                    label = 'Oddset'
                else:
                    label = label.replace('_', ' ').title()
                if self.color:
                    color = self.color
                anno.bbox_label(
                    bbox,
                    label=label,
                    color=color,
                )



from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter

from mediacatch.commands.vision import VisionCLI
from mediacatch.commands.viz import VizCLI
from mediacatch.commands.speech import SpeechCLI


def main():
    parser = ArgumentParser(
        prog='MediaCatch CLI tool',
        usage='mediacatch <command> [<args>]',
        description='MediaCatch CLI tool',
        formatter_class=ArgumentDefaultsHelpFormatter,
    )
    commands_parser = parser.add_subparsers(help='mediacatch command helpers')

    # Register commands
    SpeechCLI.register_subcommand(commands_parser)
    VisionCLI.register_subcommand(commands_parser)
    VizCLI.register_subcommand(commands_parser)

    # Let's go
    args = parser.parse_args()

    if not hasattr(args, 'func'):
        parser.print_help()
        exit(1)

    # Run
    service = args.func(args)
    service.run()


if __name__ == '__main__':
    main()



import logging

import numpy as np
from PIL import Image, ImageDraw, ImageFont

from mediacatch.utils.general import get_assets_data

logger = logging.getLogger('mediacatch.viz.annotator')

dark_colors = {
    (235, 219, 11),
    (243, 243, 243),
    (183, 223, 0),
    (221, 111, 255),
    (0, 237, 204),
    (68, 243, 0),
    (255, 255, 0),
    (179, 255, 1),
    (11, 255, 162),
}
light_colors = {
    (255, 42, 4),
    (79, 68, 255),
    (255, 0, 189),
    (255, 180, 0),
    (186, 0, 221),
    (0, 192, 38),
    (255, 36, 125),
    (104, 0, 123),
    (108, 27, 255),
    (47, 109, 252),
    (104, 31, 17),
}


def get_txt_color(
    color: tuple[int, int, int] = (128, 128, 128),
    txt_color: tuple[int, int, int] = (255, 255, 255),
) -> tuple[int, int, int]:
    if color in dark_colors:
        return (104, 31, 17)
    elif color in light_colors:
        return (255, 255, 255)
    return txt_color


class Annotator:
    def __init__(
        self,
        im: Image.Image | np.ndarray,
        line_width: int | None = 5,
        font_size: int | None = 25,
        font: str = 'Arial.ttf',
        rounded_rectangle_radius: int = 10,
    ) -> None:
        self.im = im if isinstance(im, Image.Image) else Image.fromarray(im)
        self.draw = ImageDraw.Draw(self.im)
        self.line_width = line_width or max(round(sum(im.size) / 2 * 0.006), 2)
        self.rounded_rectangle_radius = rounded_rectangle_radius

        try:
            font = get_assets_data(font)
            font_size = font_size or max(round(sum(im.size) / 2 * 0.035), 12)
            self.font = ImageFont.truetype(font, font_size)
        except Exception:
            self.font = ImageFont.load_default(font_size)

    def bbox_label(
        self,
        bbox: tuple[int, int, int, int],
        label: str | None = None,
        color: tuple[int, int, int] = (236, 56, 131),
        txt_color: tuple[int, int, int] = (255, 255, 255),
        padding: int = 5,
    ) -> None:
        txt_color = get_txt_color(color, txt_color)
        p1 = (bbox[0], bbox[1])
        bbox = (bbox[0] - padding, bbox[1] - padding, bbox[2] + padding, bbox[3] + padding)
        self.draw.rounded_rectangle(
            bbox, width=self.line_width, radius=self.rounded_rectangle_radius, outline=color
        )
        if label:
            if not label[0].isupper():
                label = label.capitalize()
            w = self.font.getlength(label)
            h = self.font.font.height
            outside = p1[1] >= h
            self.draw.rectangle(
                (
                    p1[0],
                    p1[1] - h if outside else p1[1],
                    p1[0] + w + 6,
                    p1[1] + 1 if outside else p1[1] + h + 1,
                ),
                fill=color,
            )
            self.draw.text(
                (p1[0] + 3, p1[1] if outside else p1[1] + h),
                label,
                fill=txt_color,
                font=self.font,
                anchor='ls',
            )

    def asarray(self) -> np.ndarray:
        return np.array(self.im)



import logging
import os
from typing import Any

from mediacatch.viz.annotator import Annotator
from mediacatch.viz.base_viz import BaseViz

logger = logging.getLogger('mediacatch.viz.face')


class FaceViz(BaseViz):
    """Class to visualize face detection results."""

    def __init__(
        self,
        file_path: str,
        results: list[dict[str, Any]],
        output_path: str,
        gender: bool = False,
        age: bool = False,
    ) -> None:
        """Initialize FaceViz.

        Args:
            file_path (str): File path.
            results (list[dict[str, Any]]): Results from MediaCatch Vision API.
            output_path (str): Output path.
            gender (bool, optional): Show gender. Defaults to False.
            age (bool, optional): Show age. Defaults to False.
        """
        # TODO: Add support for expression and ethnicity
        assert os.path.isfile(file_path), f'File not found: {file_path}'
        assert results, 'No face detection results found'

        self.file_path = file_path
        self.results = results
        self.results = sorted(self.results, key=lambda x: x['frame'])
        self.output_path = output_path
        self.gender = gender
        self.age = age

    def draw_results(self, anno: Annotator, frame_idx: int, width: int, height: int) -> None:
        for r in self.results:
            frame = r['frame']
            if frame == frame_idx:
                x, y, w, h = r['bbox']
                x1 = x - w / 2.0
                y1 = y - h / 2.0
                x2 = x + w / 2.0
                y2 = y + h / 2.0
                x1 = int(x1 * width)
                y1 = int(y1 * height)
                x2 = int(x2 * width)
                y2 = int(y2 * height)
                label = ''
                if self.gender:
                    label += f' {r["gender"]}'
                if self.age:
                    label += f' {r["age"]} years'
                label = label.strip()
                anno.bbox_label(
                    (x1, y1, x2, y2),
                    label=label if label else None,
                )



import json
import logging
import time
from typing import Literal

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from mediacatch import mediacatch_api_key
from mediacatch.utils import MediacatchUploadError

logger = logging.getLogger('mediacatch.vision.upload')

headers = {
    'accept': 'application/json',
    'Content-Type': 'application/json',
    'X-API-KEY': f'{mediacatch_api_key}',
}


def upload(
    fpath: str,
    type: Literal['ocr', 'face'],
    url: str = 'https://api.mediacatch.io/vision',
    fps: int | None = None,
    tolerance: int | None = None,
    min_bbox_iou: float | None = None,
    min_levenshtein_ratio: float | None = None,
    moving_threshold: int | None = None,
    max_text_length: int | None = None,
    min_text_confidence: float | None = None,
    max_text_confidence: float | None = None,
    max_height_width_ratio: float | None = None,
    get_detection_histogram: bool | None = None,
    detection_histogram_bins: int | None = None,
    max_height_difference_ratio: float | None = None,
    max_horizontal_distance_ratio: float | None = None,
    get_frame_index: bool | None = None,
    get_bbox: bool | None = None,
    face_recognition: bool | None = None,
    face_age: bool | None = None,
    face_gender: bool | None = None,
    face_expression: bool | None = None,
    face_ethnicity: bool | None = None,
    max_retries: int = 5,
    delay: float = 10.0,
    verbose: bool = True,
) -> str:
    """Upload a file to MediaCatch Vision API.

    Args:
        fpath (str): File path.
        type (Literal['ocr', 'face']): Type of inference to run on the file.
        url (str, optional): URL to the vision API. Defaults to 'https://api.mediacatch.io/vision'.
        fps (int, optional): Frames per second for video processing. Defaults to 1.
        tolerance (int, optional): Tolerance for text detection. Defaults to 10.
        min_bbox_iou (float, optional): Minimum bounding box intersection over union for merging text detection. Defaults to 0.5.
        min_levenshtein_ratio (float, optional): Minimum Levenshtein ratio for merging text detection (more info here: https://rapidfuzz.github.io/Levenshtein/levenshtein.html#ratio). Defaults to 0.75.
        moving_threshold (int, optional): If merged text detections center moves more pixels than this threshold, it will be considered moving text. Defaults to 50.
        max_text_length (int, optional): If text length is less than this value, use max_text_confidence as confidence threshold. Defaults to 3.
        min_text_confidence (float, optional): Confidence threshold for text detection (if text length is greater than max_text_length). Defaults to 0.5.
        max_text_confidence (float, optional): Confidence threshold for text detection (if text length is less than max_text_length). Defaults to 0.8.
        max_height_width_ratio (float, optional): Discard detection if height/width ratio is greater than this value. Defaults to 2.0.
        get_detection_histogram (bool, optional): If true, get histogram of detection. Defaults to False.
        detection_histogram_bins (int, optional): Number of bins for histogram calculation. Defaults to 8.
        max_height_difference_ratio (float, optional): Determine the maximum allowed difference in height between two text boxes for them to be merged. Defaults to 0.5.
        max_horizontal_distance_ratio (float, optional): Determine if two boxes are close enough horizontally to be considered part of the same text line. Defaults to 0.9.
        get_frame_index (bool, optional): If true, get frame index. Defaults to None.
        get_bbox (bool, optional): If true, get bounding box. Defaults to None.
        face_recognition (bool, optional): If true, run face recognition. Defaults to None.
        face_age (bool, optional): If true, get face age. Defaults to None.
        face_gender (bool, optional): If true, get face gender. Defaults to None.
        face_expression (bool, optional): If true, get face expression. Defaults to None.
        face_ethnicity (bool, optional): If true, get face ethnicity. Defaults to None.
        max_retries (int, optional): Maximum number of retries. Defaults to 5.
        delay (float, optional): Delay between retries. Defaults to 10.0.
        verbose (bool, optional): If True, print log messages. Defaults to True.

    Returns:
        str: File ID.
    """
    if verbose:
        logger.info(f'Uploading file {fpath} to MediaCatch Vision API')

    extra = {}
    if fps is not None:
        extra['fps'] = fps
    if tolerance is not None:
        extra['tolerance'] = tolerance
    if min_bbox_iou is not None:
        extra['min_bbox_iou'] = min_bbox_iou
    if min_levenshtein_ratio is not None:
        extra['min_levenshtein_ratio'] = min_levenshtein_ratio
    if moving_threshold is not None:
        extra['moving_text_threshold'] = moving_threshold
    if max_text_length is not None:
        extra['max_text_length'] = max_text_length
    if min_text_confidence is not None:
        extra['min_text_confidence'] = min_text_confidence
    if max_text_confidence is not None:
        extra['max_text_confidence'] = max_text_confidence
    if max_height_width_ratio is not None:
        extra['max_height_width_ratio'] = max_height_width_ratio
    if get_detection_histogram is not None:
        extra['get_detection_histogram'] = get_detection_histogram
    if detection_histogram_bins is not None:
        extra['detection_histogram_bins'] = detection_histogram_bins
    if max_height_difference_ratio is not None:
        extra['max_height_difference_ratio'] = max_height_difference_ratio
    if max_horizontal_distance_ratio is not None:
        extra['max_horizontal_distance_ratio'] = max_horizontal_distance_ratio
    if get_frame_index is not None:
        extra['frame'] = get_frame_index
    if get_bbox is not None:
        extra['bbox'] = get_bbox
    if face_recognition is not None:
        extra['recognition'] = face_recognition
    if face_age is not None:
        extra['age'] = face_age
    if face_gender is not None:
        extra['gender'] = face_gender
    if face_expression is not None:
        extra['expression'] = face_expression
    if face_ethnicity is not None:
        extra['ethnicity'] = face_ethnicity

    # Get presigned URL
    data = {'filename': fpath, 'type': type, 'extra': extra}
    response = requests.post(f'{url}/upload/', headers=headers, data=json.dumps(data))
    if response.status_code != 201:
        raise MediacatchUploadError(f'Failed to get presigned URL: {response}')

    presigned_url_response = response.json()
    file_id = presigned_url_response['file_id']

    # Create a session with retry logic
    session = requests.Session()
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[500, 502, 503, 504, 400])
    session.mount('https://', HTTPAdapter(max_retries=retries))

    # upload to s3
    success = False
    for _ in range(max_retries):
        try:
            with open(fpath, 'rb') as f:
                files = {'file': (presigned_url_response['fields']['key'], f)}
                fields_dict = dict(presigned_url_response['fields'])
                response = session.post(
                    presigned_url_response['url'],
                    data=fields_dict,
                    files=files,
                    verify=True,
                )

            if response.status_code != 204:
                raise MediacatchUploadError(f'Failed to upload to S3: {response}')

            success = True
            break

        except Exception:
            success = False
            time.sleep(delay)

    if not success:
        raise MediacatchUploadError(f'Failed to upload to S3: {response}')

    # Mark file as uploaded
    data = {'file_id': file_id}
    response = requests.post(f'{url}/upload/complete/', headers=headers, data=json.dumps(data))
    if response.status_code != 200:
        raise MediacatchUploadError(f'Failed to mark file {fpath} as uploaded: {response}')

    if verbose:
        logger.info(f'File {fpath} uploaded with ID {file_id}')

    return file_id



class MediacatchError(Exception):
    """
    Base class for all Mediacatch exceptions.
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class MediacatchAPIError(Exception):
    """
    Raised when the API returns an error status code.
    """

    def __init__(self, status_code, reason):
        self.status_code = status_code
        self.reason = reason
        super().__init__(f'API returned error status code: {status_code} with reason: {reason}')


class MediacatchTimeoutError(Exception):
    """
    Raised when a timeout occurs during API requests.
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class MediacatchUploadError(Exception):
    """
    Raised when an error occurs during file upload.
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



import logging
import os
from abc import ABC, abstractmethod
from typing import Any

import cv2
from PIL import Image
from tqdm.auto import tqdm

from mediacatch.viz.annotator import Annotator

logger = logging.getLogger('mediacatch.viz')


class BaseViz(ABC):
    """Base class for visualization vision results."""

    file_path: str
    results = list[dict[str, Any]]
    output_path: str

    @abstractmethod
    def draw_results(self) -> None:
        raise NotImplementedError()

    def create_viz(self) -> None:
        """Create visualization."""
        assert os.path.isfile(self.file_path), f'File not found: {self.file_path}'
        assert self.results, 'No OCR results found'
        assert self.output_path, 'Output path is not defined'

        fname = os.path.basename(self.file_path)
        cap = cv2.VideoCapture(self.file_path)
        if not cap.isOpened():
            logger.error(f'Failed to open video: {self.file_path}')
            cap.release()
            return None

        num_frames = max(1, int(cap.get(cv2.CAP_PROP_FRAME_COUNT)))
        input_fps = float(max(1, cap.get(cv2.CAP_PROP_FPS)))
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        fourcc = cv2.VideoWriter_fourcc(*'XVID')
        video = cv2.VideoWriter(self.output_path, fourcc, input_fps, (width, height))

        failed_frames = 0
        try:
            for i in tqdm(range(num_frames), desc='Creating video'):
                ret, frame = cap.read()
                if not ret:
                    if failed_frames > 10:
                        logger.error(f'Failed to read frames for {failed_frames} times')
                        break
                    logger.warning(f'Failed to read frame {i} from {fname}')
                    failed_frames += 1
                    continue

                width = frame.shape[1]
                height = frame.shape[0]
                anno = Annotator(Image.fromarray(frame))
                self.draw_results(anno, i, width, height)
                video.write(anno.asarray())

        except KeyboardInterrupt:
            logger.warning('User interrupted the process')

        except Exception as e:
            logger.error(f'An error occurred: {e}')

        finally:
            video.release()
            cap.release()
            logger.info(f'Vizualization is saved to {self.output_path}')



from mediacatch.viz.detect import DetectViz
from mediacatch.viz.face import FaceViz
from mediacatch.viz.ocr import OCRViz
from mediacatch.viz.speech import SpeechViz

__all__ = [
    'DetectViz',
    'FaceViz',
    'OCRViz',
    'SpeechViz',
]



import json
import logging
import os
from argparse import ArgumentParser
from typing import Literal

from mediacatch.commands import BaseCLICommand
from mediacatch.viz import DetectViz, FaceViz, OCRViz, SpeechViz

logger = logging.getLogger('mediacatch.cli.viz')


def viz_cli_factory(args):
    return VizCLI(
        type=args.type,
        file_path=args.file_path,
        results_path=args.results_path,
        output_path=args.output_path,
        bgr_color=tuple(args.bgr_color) if args.bgr_color else None,
        detect_classes=args.detect_classes,
        detect_min_conf=args.detect_min_conf,
        ocr_max_text_length=args.ocr_max_text_length,
        face_gender=args.face_gender,
        face_age=args.face_age,
        speech_no_subtitles=args.speech_no_subtitles,
        speech_no_meta=args.speech_no_meta,
    )


class VizCLI(BaseCLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        viz_parser: ArgumentParser = parser.add_parser(
            'viz', help='CLI tool to visualize the results of MediaCatch'
        )
        viz_parser.add_argument(
            'type',
            type=str,
            choices=['ocr', 'face', 'detect', 'speech'],
            help='Type of inference to visualize',
        )
        viz_parser.add_argument('file_path', type=str, help='Path to the file to visualize')
        viz_parser.add_argument('results_path', type=str, help='Path to the results json file')
        viz_parser.add_argument(
            'output_path',
            type=str,
            help='Path to save the output visualization',
        )
        viz_parser.add_argument(
            '--bgr-color',
            type=int,
            nargs=3,
            default=None,
            help='BGR color to use for the visualization',
        )

        # Detect specific arguments (must start with --detect-)
        viz_parser.add_argument(
            '--detect-classes',
            type=str,
            nargs='+',
            default=None,
            help='Classes to visualize',
        )
        viz_parser.add_argument(
            '--detect-min-conf',
            type=float,
            default=0.5,
            help='Minimum confidence to consider a detection',
        )

        # OCR specific arguments (must start with --ocr-)
        viz_parser.add_argument(
            '--ocr-max-text-length',
            type=int,
            default=30,
            help='Maximum length of text to display',
        )

        # Face specific arguments (must start with --face-)
        viz_parser.add_argument(
            '--face-gender',
            action='store_true',
            help='Vizualize gender detection results',
        )
        viz_parser.add_argument(
            '--face-age',
            action='store_true',
            help='Vizualize age detection results',
        )

        # Speech specific arguments (must start with --speech-)
        viz_parser.add_argument(
            '--speech-no-subtitles',
            action='store_true',
            help='Do not create subtitles for speech',
        )
        viz_parser.add_argument(
            '--speech-no-meta',
            action='store_true',
            help='Do not create meta subtitles for speech',
        )

        viz_parser.set_defaults(func=viz_cli_factory)

    def __init__(
        self,
        type: Literal['ocr', 'face', 'detect'],
        file_path: str,
        results_path: str,
        output_path: str,
        bgr_color: tuple[int, int, int] | None = None,
        detect_classes: list[str] | None = None,
        detect_min_conf: float = 0.5,
        ocr_max_text_length: int = 30,
        face_gender: bool = False,
        face_age: bool = False,
        speech_no_subtitles: bool = False,
        speech_no_meta: bool = False,
    ) -> None:
        assert os.path.isfile(file_path), f'File not found: {file_path}'
        assert os.path.isfile(results_path), f'Results file not found: {results_path}'

        self.type = type
        self.file_path = file_path
        self.results_path = results_path
        self.output_path = output_path
        self.bgr_color = bgr_color
        self.detect_classes = detect_classes
        self.detect_min_conf = detect_min_conf
        self.ocr_max_text_length = ocr_max_text_length
        self.face_gender = face_gender
        self.face_age = face_age
        self.speech_no_subtitles = speech_no_subtitles
        self.speech_no_meta = speech_no_meta

        with open(results_path, 'r') as f:
            self.results = json.load(f)

    def run(self) -> None:
        if self.type == 'face':
            logger.info('Creating face visualization')
            viz = FaceViz(
                file_path=self.file_path,
                results=self.results,
                output_path=self.output_path,
                gender=self.face_gender,
                age=self.face_age,
            )
        elif self.type == 'detect':
            logger.info('Creating detection visualization')
            viz = DetectViz(
                file_path=self.file_path,
                results=self.results,
                output_path=self.output_path,
                color=self.bgr_color,
                classes=self.detect_classes,
                min_conf=self.detect_min_conf,
            )
        elif self.type == 'ocr':
            logger.info('Creating OCR visualization')
            viz = OCRViz(
                file_path=self.file_path,
                results=self.results,
                output_path=self.output_path,
                max_text_length=self.ocr_max_text_length,
            )
        elif self.type == 'speech':
            logger.info('Creating speech visualization')
            viz = SpeechViz(
                file_path=self.file_path,
                results=self.results,
                output_path=self.output_path,
                subtitles=not self.speech_no_subtitles,
                meta=not self.speech_no_meta,
            )
        else:
            raise NotImplementedError(f'Visualization for {self.type} is not implemented')

        viz.create_viz()



from mediacatch.utils.exceptions import (
    MediacatchAPIError,
    MediacatchError,
    MediacatchTimeoutError,
    MediacatchUploadError,
)
from mediacatch.utils.general import (
    check_ffmpeg,
    compress_to_ogg,
    get_assets_data,
    get_data_from_url,
    load_data_from_json,
    make_request,
    read_file_in_chunks,
)

__all__ = [
    'get_data_from_url',
    'load_data_from_json',
    'get_assets_data',
    'make_request',
    'read_file_in_chunks',
    'MediacatchAPIError',
    'MediacatchError',
    'MediacatchTimeoutError',
    'MediacatchUploadError',
    'check_ffmpeg',
    'compress_to_ogg',
]



import json
import logging
import os
import subprocess
import time
from importlib.resources import files
from io import BufferedReader
from typing import Generator

import requests

from mediacatch.utils import MediacatchUploadError

logger = logging.getLogger('mediacatch.utils.general')


def get_data_from_url(url: str) -> dict:
    """Get data from a URL.

    Args:
        url (str): The URL to get data from.

    Returns:
        dict: Data dictionary from url.
    """
    logger.info(f'Getting data from {url}')
    response = requests.get(url)
    assert response.status_code == 200, f'Failed to get data from {url}: {response}'
    return response.json()


def load_data_from_json(json_file: str) -> dict:
    """Load data from a JSON file.

    Args:
        json_file (str): JSON file with data.

    Returns:
        dict: Data dictionary from JSON file.
    """
    logger.info(f'Loading data from {json_file}')
    assert os.path.isfile(json_file), f'File {json_file} does not exist'
    with open(json_file, 'r') as f:
        result = json.load(f)
    return result


def get_assets_data(fname: str) -> str:
    data_file = files('mediacatch').joinpath(f'assets/{fname}')
    assert data_file.is_file(), f'File {fname} does not exist in mediacatch/assets folder'
    return data_file


def make_request(
    method: str,
    url: str,
    headers: dict[str, str],
    max_retries: int = 3,
    delay: float = 0.5,
    **kwargs,
) -> requests.Response:
    """Makes a request to the given URL.

    Args:
        method (str): Request method.
        url (str): The URL to make the request to.
        headers (dict[str, str]): Headers to include in the request.
        max_retries (int, optional): Maximum retries. Defaults to 3.
        delay (float, optional): Delay between retries. Defaults to 0.5.

    Raises:
        RuntimeError: If method is post and response status code is equal or greater than 400.
        RuntimeError: If maximum retries limit is reached.

    Returns:
        requests.Response: Response object.
    """
    for _ in range(max_retries):
        response = getattr(requests, method)(url, headers=headers, **kwargs)
        if 200 <= response.status_code < 300:
            return response

        if method == 'post' and response.status_code >= 400:
            raise RuntimeError(f'Error during request to {url}', response.json()['detail'])

        time.sleep(delay)

    raise RuntimeError('Maximum retry limit reached for request', None)


def read_file_in_chunks(
    file_: BufferedReader, chunk_size: int = 100 * 1024 * 1024
) -> Generator[bytes, None, None]:
    """Reads a file in chunks.

    Args:
        file_ (BufferedReader): File to read.
        chunk_size (int, optional): Size . Defaults to 100*1024*1024.

    Yields:
        Generator[bytes, None, None]: Chunk of the file.
    """
    while chunk := file_.read(chunk_size):
        yield chunk


def check_ffmpeg() -> None:
    try:
        subprocess.run(['ffmpeg', '-version'], check=True, capture_output=True)
    except FileNotFoundError:
        raise MediacatchUploadError(
            'FFMPEG is not installed or not in the system PATH\nPlease install ffmpeg before continuing.\n\tOn Ubuntu/Debian: sudo apt install ffmpeg\n\tOn macOS with Homebrew: brew install ffmpeg\n\tOn Windows: https://ffmpeg.org/download.html'
        )


def compress_to_ogg(input_path: str, output_path: str, sample_rate: int = 16000) -> None:
    """Compresses a file to OGG format.

    Args:
        input_path (str): Input file path.
        output_path (str): Output file path.
        sample_rate (int, optional): Sample rate. Defaults to 16.

    Raises:
        MediacatchUploadError: If failed to compress file to OGG format.
    """
    try:
        command = [
            'ffmpeg',
            '-hide_banner',
            '-loglevel',
            'error',
            '-nostats',
            '-i',
            f'{input_path}',
            '-vn',
            '-ar',
            f'{sample_rate}',
            '-ac',
            f'{1}',
            '-c:a',
            'libvorbis',
            '-y',
            f'{output_path}',
        ]
        subprocess.run(command, check=True, capture_output=True, text=True)
    except subprocess.CalledProcessError as e:
        raise MediacatchUploadError(f'Failed to compress file to OGG format, got {e.stderr}')
